'''Basic action classes.'''

from concepts.color import *
from concepts.geometry import *


class Action(object):
    def __init__(self, person):
        self._person = person
    
    def person(self):
        return self._person
    
    def displacement(self):
        return Vector(0, 0)
    
    def vector(self):
        return Vector(0, 0)
    
    def remaining_vector(self):
        return Vector(0, 0)
    
    def done_vector(self):
        return Vector(0, 0)
    
    def done(self):
        self.person().done_acting()
    
    def plan(self):
        return OneActionPlan(self)
    
    def act(self):
        pass

    def __str__(self):
        return self.__class__.__name__
                
            
class ProgressingAction(Action):
    def __init__(self, person):
        super(ProgressingAction, self).__init__(person)
        
        self._progress = 0.0
    
    def speed(self):
        return 0.01
#    
    def repeating(self):
        return False
    
    def progress(self):
        return self._progress
    
    def progress_added(self):
        pass
    
    def progress_full(self):
        pass
        
    def act(self):
        self._progress += self.speed()
        self.progress_added()
        
        while self.progress() >= 1.0:
            self.progress_full()
            
            if self.repeating():
                self._progress -= 1.0
            else:
                self.done()
                return


class Plan(object):
    def __init__(self, person):
        self._person = person
    
    def color(self):
        return Color(255, 255, 0)
        
    def person(self):
        return self._person
    
    def next_action(self):
        pass
    
    def path(self):
        return []

    def __str__(self):
        return self.__class__.__name__


class OneActionPlan(Plan):
    def __init__(self, action, color = Color(128, 128, 0)):
        super(OneActionPlan, self).__init__(action.person())
        
        self._action = action
        self._color  = color
        
    def color(self):
        return self._color
    
    def next_action(self):
        action = self._action
        self._action = None
        
        return action

        