'''All things gluttony.'''

from concepts.color import *
            
from move import *


class Consumption(Action):
    def __init__(self, person):
        super(Consumption, self).__init__(person)
        
        self._progress = 0.0
    
    def speed(self):
        return self.good().consumption_speed()
    
    def good(self):
        return self.person().backpack()
    
    def progress(self):
        return self._progress
    
    def unit_consumed(self):
        self.good().consumed_by(self.person(), 1)
    
    def act(self):
        self._progress += self.speed()
        
        while self.progress() >= 1.0:
            self._progress -= 1.0
            self.unit_consumed()
            self.done()


class GoAndConsume(Plan):
    def __init__(self, person, number, predicate):
        super(GoAndConsume, self).__init__(person)
        
        self._number    = number
        self._predicate = predicate
        self._route     = None
    
    def color(self):
        return Color(255, 255, 0)
        
    def number(self):
        return self._number
    
    def predicate(self):
        return self._predicate
    
    def matches(self, x):
        return self.predicate()(x)
    
    def route(self):
        return self._route
    
    def action(self):
        return self._action
    
    def next_action(self):
        if self.route() is not None:
            action = self.route().next_action()
        
            if action is not None:
                return action
        
        if self.number() <= 0:
            return None
        
        if self.person().backpack() is None:
            tile = self.person().tile()
            
            if tile.has_goods() and self.matches(tile.goods()):
                self.person().take(tile.goods().with_number(self.number()))
            else:
                tile = self.person().find_matching_resource(self.predicate())
                
                if tile is None:
                    return None
                
                self._route = self.person().route_to(tile)
        elif self.matches(self.person().backpack()):
            self._number -= 1
            
            return Consumption(self.person())
        else:
            self.person().drop()
            
        return self.next_action()

    def path(self):
        if self.route() is None:
            return []
        
        return self.route().path()
