'''Yummy yummy goods produced riverside.'''

from concepts.color import *


class Goods(object):
    def __init__(self, number):
        self._number    = number
        self._callbacks = {}
        
    @classmethod
    def name(self):
        return ('Good', 'Goods')
    
    @classmethod
    def color(self):
        return Color(255, 0, 0)
    
    @classmethod
    def singular(self):
        return self.name()[0]
    
    @classmethod
    def plural(self):
        return self.name()[1]
    
    @classmethod
    def is_edible(self):
        return False
    
    @classmethod
    def consumption_speed(self):
        return 0.2
    
    def with_number(self, number):
        return self.__class__(number)
    
    def consumption_effect(self, person, number):
        pass
    
    def consumed_by(self, person, number):
        number = min(self.number(), number)
        
        self.decrease(number)
        self.consumption_effect(person, number)
        
    def number(self):
        return self._number
    
    def is_full(self):
        return (self._number >= 25)
    
    def increase(self, number):
        if self.number() <= 0:
            return
        
        self._number += number
        
        if self.number() <= 0:
            self.destroy()

        if self.number() > 25:
            self._number = 25
            
    def decrease(self, number):
        self.increase(-number)
        
    def destroy(self):
        self._number = 0
        
        for x in self._callbacks.values():
            x(self)
    
    def can_take(self, other):
        if not self.matches(other):
            return 0
        
        return min(25 - self.number(), other.number())
    
    def on_destruction(self, name, callback):
        self._callbacks[name] = callback
    
    def remove_callback(self, name):
        del self._callbacks[name]
        
    def is_at_least(self, resource):
        return (self.__class__ is resource.__class__ and 
                self.number() >= resource.number())
    
    def matches(self, resource):
        return (self.__class__ is resource.__class__)
    
    def copy(self):
        return self.with_number(self.number())
    
    def take(self, number):
        if number >= self.number():
            number = self.number()
        
        if number <= 0:
            return None
                
        self.decrease(number)
        
        return self.__class__(number)

    def numerus(self):
        if self.number() == 1:
            return self.singular()
        else:
            return self.plural()
        
    def __str__(self):
        return '%d %s' % (self.number(), self.numerus().lower())


class ResourceOverview(object):
    def __init__(self, resources):
        self._resources = resources
        self._numbers   = {}
        self._types     = []
        self._total     = 0
        
        for x in resources:
            num  = self._numbers.get(x.__class__, 0)
            num += x.number()
            self._total += x.number()
            self._numbers[x.__class__] = num
        
        self._types = list(self._numbers.keys())
        self._types.sort(lambda a, b: cmp(a.__name__, b.__name__))
        
    def resources(self):
        return self._resources
    
    def types(self):
        return list(self._types)
    
    def number_of(self, type):
        return self._numbers.get(type, 0)
    
    def total(self):
        return self._total
    
    def is_empty(self):
        return (self._total == 0)
    
    def list(self):
        list = []
        
        for type in self.types():
            list.append(type(self.number_of(type)))
        
        return list
    
    
class Food(Goods):
    @classmethod
    def name(self):
        return ('Food', 'Food')
    
    @classmethod
    def is_edible(self):
        return True
    
    @classmethod
    def nutrional_value(self):
        return 0.01
    
    @classmethod
    def happyness_value(self):
        return 0.01

    def consumption_effect(self, person, number):
        person.nutrition().add(number * self.nutrional_value())
        
        for _ in range(number):
            person.happiness().add(self.happiness_value() *
                                   (1 - person.happiness().amount()))
    
    
class Wheat(Food):
    @classmethod
    def name(self):
        return ('Wheat', 'Wheat')
    
    @classmethod
    def color(self):
        return Color(255, 255, 0)
    
    @classmethod
    def nutrional_value(self):
        return 0.1
    
    @classmethod
    def happiness_value(self):
        return 0.01

    @classmethod
    def consumption_speed(self):
        return 0.05

    
class Grapes(Food):
    @classmethod
    def name(self):
        return ('Grape', 'Grapes')
    
    @classmethod
    def color(self):
        return Color(210, 156, 255)
    
    @classmethod
    def nutrional_value(self):
        return 0.05
    
    @classmethod
    def happiness_value(self):
        return 0.05

    @classmethod
    def consumption_speed(self):
        return 0.1

    
class Wine(Food):
    @classmethod
    def name(self):
        return ('Wine', 'Wine')
    
    @classmethod
    def color(self):
        return Color(192, 0, 255)
    
    @classmethod
    def nutrional_value(self):
        return 0.01
    
    @classmethod
    def happiness_value(self):
        return 0.2

    @classmethod
    def consumption_speed(self):
        return 0.3
    
    
class Wood(Goods):
    @classmethod
    def name(self):
        return ('Wood', 'Wood')
    
    @classmethod
    def color(self):
        return Color(128, 96, 0)

    
class Stone(Goods):
    @classmethod
    def name(self):
        return ('Stone', 'Stone')
    
    @classmethod
    def color(self):
        return Color(96, 96, 96)
    
    
class Marble(Goods):
    @classmethod
    def name(self):
        return ('Marble', 'Marble')
    
    @classmethod
    def color(self):
        return Color(255, 255, 255)

    
class Ore(Goods):
    @classmethod
    def name(self):
        return ('Ore', 'Ore')
    
    @classmethod
    def color(self):
        return Color(192, 32, 32)

    
class Coal(Goods):
    @classmethod
    def name(self):
        return ('Coal', 'Coal')
    
    @classmethod
    def color(self):
        return Color(48, 48, 48)

    
class Iron(Goods):
    @classmethod
    def name(self):
        return ('Iron', 'Iron')
    
    @classmethod
    def color(self):
        return Color(148, 148, 220)

    
class Sapphire(Goods):
    @classmethod
    def name(self):
        return ('Sapphire', 'Sapphires')
    
    @classmethod
    def color(self):
        return Color(64, 64, 255)
