'''Beware stupid gods!'''

from concepts.color import *
from concepts.geometry import *

from river.building import *
from river.goods import *
from river.terrain import *

from river.actions.jobs import *

import random
import operator
    

class God(object):
    def __init__(self, city):
        self._city   = city
        self._anger  = 0.2
        self._danger = 0.0
    
    def city(self):
        return self._city
    
    def number_temples(self):
        return len(self.city().buildings_of_type(Temple))

    def anger(self):
        return self._anger
    
    def danger(self):
        return self._danger
    
    def increase_anger(self, value):
        self._anger   = min(1.0, self._anger + value)
        self._danger += value * self._anger * self._anger * 3
        
        if self.danger() >= 1.0:
            self._danger -= 1.0
            self.do_evil_stuff()
    
    def reduce_anger(self, factor):
        self._anger *= 1 - factor
        
    def new_temple(self):
        if self.number_temples() == 0:
            self.city().time().do_every(1, lambda: self.increase_anger(0.0003))
        elif self.number_temples() == 2:
            def mountain(x):
                return (x.terrain().__class__ is Mountain)
            
            tiles = filter(lambda x: all(map(mountain, [x] + list(x.neighbors()))) and
                           x.building() is None, self.city().map().tiles())
            
            if tiles:
                tile = random.choice(tiles)
                tile.become(MagicalMountain)
                self.city().log('"OOOOH, A THIRD TEMPLE! LET ME SHOW YOU SOMETHING MAGIAL, LIKE I PROMISED!"', Color(192, 0, 0))
                self.city().log('"THERE ARE POWERFUL MAGICAL BLUE STONES IN THIS MOUNTAIN! THEY ARE USELESS TO ME!"', Color(192, 0, 0))
                self.city().log('"BUT THEY ARE USELESS TO YOU, TOO! HAHAHA!"', Color(255, 0, 0))
                
    def do_evil_stuff(self):
        city      = self.city()
        color     = Color(192, 0, 0)
        resources = ResourceOverview(city.goods())

        if random.random() < 0.3:
            city.change_attributes(-3)
            city.log('"YOU FOOLS!"', color)
        elif random.random() < 0.7 or not resources.types():
            city.change_happiness(-0.5)
            city.log('"YOU SHALL ALL SUFFER!"', color)
        else:
            resource = random.choice(resources.types())
            name     = resource.plural().upper()
            
            city.destroy_resource(resource)
            city.log('"ALL YOUR %s ARE BELONG TO US!"' % name, color)
    
    
class Temple(Building):
    def __init__(self, city, tile):
        super(Temple, self).__init__(city, tile, [Priest(self),
                                                  Priest(self),
                                                  Priest(self),
                                                  Priest(self),
                                                  Priest(self)])
        
        self.god().reduce_anger(0.5)
        self.god().new_temple()
    
    def anger(self):
        return self.god().anger()

    def danger(self):
        return self.god().danger()
    
    def god(self):
        return self.city().god()
    
    @classmethod
    def name(cls):
        return 'Temple'
    
    @classmethod
    def size(self):
        return 5
    
    @classmethod
    def cost(self):
        return [Stone(49), Marble(21), Iron(7)]
    
    @classmethod
    def color(self):
        return Color(64, 0, 0)
    
    @classmethod
    def difficulty(self):
        return 1
            
                        
Building.add_type(Temple, 'living', 3)


class Priest(Job):
    def __init__(self, temple):
        super(Priest, self).__init__(temple)
        
    def title(self):
        return 'Priest'
    
    def color(self):
        return Color(128, 0, 0)
    
    def body_weight(self):
        return 0
    
    def mind_weight(self):
        return 0
    
    def spirit_weight(self):
        return 1
    
    def work_speed(self):
        return 0.01

    def work_done(self):
        self.building().god().reduce_anger(0.1 * self.performance())
        
    def target(self):
        if random.random() < 0.25:
            return self.office()
        else:
            return random.choice(list(self.building().tiles()))
    

class Mirror(ProcessingBuilding):
    def __init__(self, city, tile):
        super(Mirror, self).__init__(city, tile, [Seer(self)], [Sapphire(7)])
    
    @classmethod
    def name(cls):
        return 'Mirror'
    
    @classmethod
    def size(self):
        return 1
    
    @classmethod
    def cost(self):
        return [Sapphire(1)]

    @classmethod
    def built_on(cls):
        return [Waterfall]
    
    @classmethod
    def color(cls):
        return Color(0, 0, 255)
    
    @classmethod
    def difficulty(cls):
        return 0.5
    
    @classmethod
    def max_buildable(cls):
        return 1
    
    @classmethod
    def too_many(cls):
        return 'The new mirror shatters! It appears only one can hold contact to Aqua\'s magic.'
    
    def resource_added(self, x):
        for _ in range(x.number()):
            self.do_good_stuff()
                
    def do_good_stuff(self):
        city        = self.city()
        color       = Color(0, 0, 255)
        performance = self.jobs()[0].performance()
        
        city.god().increase_anger(0.20)
        
        if random.random() < 0.3:
            if random.random() < performance / 2:
                city.change_attributes(2)
            else:
                city.change_attributes(1)
                
            city.log('"Be blessed, my friends!"', color)
        elif random.random() < 0.5:
            city.change_happiness(0.10 + 0.20 * performance)
            city.log('"Let me cheer you all up!"', color)
        elif random.random() < 0.5:
            city.change_social(0.25 + 0.25 * performance)
            city.log('"May you all never be lonely!"', color)
        else:
            city.change_nutrition(0.25 + 0.25 * performance)
            city.log('"My mana will feed you all!"', color)
    
                        
Building.add_type(Mirror, 'living', 4)


class Seer(ProcessingJob):
    def title(self):
        return 'Seer'
    
    def color(self):
        return Color(64, 216, 255)
    
    def body_weight(self):
        return 0
    
    def mind_weight(self):
        return 0
    
    def spirit_weight(self):
        return 1
    
    def work_speed(self):
        return 0.0005
    