'''The storage industry.'''

from river.building import *
from river.actions.jobs import *
from river.goods import *

import random


class Storage(Building):
    def __init__(self, city, tile):
        super(Storage, self).__init__(city, tile, [Collecting(self),
                                                   Collecting(self)])
    
        self._forbidden = set()
        
    @classmethod
    def name(cls):
        return 'Storage'
        
    @classmethod
    def size(self):
        return 3
    
    @classmethod
    def cost(self):
        return [Wood(3), Stone(7)]
    
    @classmethod
    def color(self):
        return Color(192, 192, 255)

    def policy_of(self, x):
        return not (x in self._forbidden)

    def allow(self, x):
        self._forbidden.discard(x)
        
    def forbid(self, x):
        self._forbidden.add(x)
        
        
Building.add_type(Storage, 'infra', 4)


class Collecting(Job):
    def __init__(self, storage):
        super(Collecting, self).__init__(storage)
        
    def title(self):
        return 'Collector'
    
    def color(self):
        return Color(128, 128, 255)
    
    def body_weight(self):
        return 1
    
    def mind_weight(self):
        return 3
    
    def spirit_weight(self):
        return 0
    
    def plan(self):
        failure = (random.random() < (1 - self.performance()) * 0.05)
        
        def unstored(r, t):
            building = t.building()
            resource = t.goods()
            
            return (resource is not None and (building is None or 
                    building.__class__ not in [Storage, ConstructionSite] or
                    t == self.building().office()) and
                    (self.building().policy_of(resource.__class__) or
                    (failure and random.random() < 0.1)))

        p = self.person()

        if p.is_at(self.office()):
            return FetchResource(p, self.target(), None, unstored)
        else:
            return p.route_to(self.office())
            