'''An epic battle, of sorts.'''

from river.scenario import *

from despair import *


class Battle(Scenario):        
    @classmethod
    def name(cls):
        return 'The End'

    def city_name(self):
        return 'Iron Mountain'
    
    def starting_tile(self):
        return self.map().tile(49, 5)
    
    def layout(self):
        return ['...%%%%%....%%...%*%.....~~~~~~~~~.....$$$.$$=.$$.',
                '..........*%.%%.......~~~~~~~~~~~~~~~...$=$..$$$$$',
                '..%%%...%%...%..~~~~~~~~~~~~~~~~~~~~~~~~..........',
                '.%%*..%*%....~~~~~~~~~~~~~~......~~~~~~~~..$=$....',
                '.....%%~~~~~~~~~~~~~~~...............~~~~~..$=$...',
                '%%....~~~~~~~~~~~........$..$...$=$..~~~~~...rrrrr',
                '..%%%~~~~..%%%.....%.......=$....$$$~~~~..=$......',
                '.%*.~~~~.%%*%.....%...$.........=$.~~~~~...$$$....',
                '.%%.~~~~....%%%.............%%*...~~~~~........$..',
                '.%..~~~~...%%*...%%*%....$$..*%..~~~~..$=$.$=$.=$$',
                '....%~~~~~.....*%%%.^^^^..$=$...~~~~~....$...$....',
                '..%%..~~~~~~..%%..^^^^^^^.$$..~~~~~...$$.....$$...',
                '.%*%....~~~~~...^^^^^^^^^^^^.~~~~~......%*%.......',
                '..%%%...~~~~~.^^^^^^^^^^^^^^^^^~~~~~%%....$=$.$...',
                '......~~~~~~.^^^^^^^^^^^^^^^^^^^^~~~~*%%......$$..',
                '%..~~~~~~~~.^^^^^^^^^^^^^^^^^^^^^^~~~%%*%%..$$=$..',
                '++~~~~~~~~~^^^^^^^^^^^^^^^^^^^^^~~~~~~~%%*.....$..',
                '.~~~~~~^^^^^^^^^^^^^^^^^^^^^^^^~~~~~~~~~~~%.......',
                '.~~~~~^^^^^^^^^^^^^^^^^^^^^^^^~~~^^^^^~~~~~~.=$$$.',
                '.~~~~^^^^^^^^^^^^^^^^^^^^^^^^^###^^^^^^^~~~~~..=$$',
                '.+~~~~~^^^^^^^^^^^^^^^^^^^^^^^###^^^^^^^^.~~~~....',
                '%%.~~~~~+++^^^^^^^^^^^^^^^^^^^###^^^^^^...~~~~.$$$',
                '..~~~~~++++++^^^^^^^^^^^^^^^^^###^^^^.$..~~~~..=$.',
                '~~~~~%%%+++++%*%%^^^^^^^^^^^^^###.......~~~~......',
                '~~~%%*%%%%++++%%%%^^^^^^^^^^^^~~~..$$...~~~...$...',
                '~~..%%%%%%%%%*%%%...^^^^^^^%%.~~~......~~~~..$=...',
                '......%%%%%%%%%%....%^^^^%%*...~~~..$..~~~~~~~~~~~',
                '........%%*%%%%....%*%%......~~~........~~~~~~~~~~',
                '..*%%....%%%%.......%*%%...~~~...$$$.$$..~~~~~~~~~',
                '.%%%%%........%%*.....%%..~~~...$=$...$...........',
                '...%*%%......%%%%%........~~~...............$$..$.',
                '...............*%%..%%%....~~~~....$$...$$......=$',
                '...................%*%.......~~~~.............$.$.',
                '........................%%*....~~~~......$$.$.....',
                '.......................%%%%%....~~~.$$$......$=$..',
                '........................*%%.....~~~$=$$$..........',
                '...............................~~~~...$$$...$=$...',
                '..............................~~~~..$........$..$.',
                '..............................~~~.$$......$$....$$']

    def setup(self):
        for x in [Road, Bridge, Plaza, Storage, House, School, Temple, 
                  Farm, Sawmill, Quarry, Mine, Vineyard, Winery, Oven, Furnace]:
            self.allow(x)
            
        self.create_immigrants(4)
        
    def intro(self):
        self.flavor(['"BUILD ME A BIG TEMPLE HERE!" he commanded upon arriving at this ',
                     'desolate location. "OUT OF IRON'])
        self.flavor(['AND MARBLE!" "Okay, we will set up an iron industry. It is well known that there ',
                     'is no coal in this mountain.'])
        self.flavor(['That means we will need to burn wood to get charcoal to smelt the ore."'])
        
        self.objective(['Build a temple.'])
        
    def script(self):
        total_p     = lambda: len(self.city().persons())
        number_of_p = lambda x: len(self.city().persons_of_type(x))
        number_of_b = lambda x: len(self.city().buildings_of_type(x))
        number_of_r = lambda x: self.city().resources_of_type(x)
        number_of_h = lambda: len(self.city().persons_with(lambda: x.happiness().is_happy()))
        
        def max_power():
            mirror = self.city().buildings_of_type(Mirror)
            
            if not mirror:
                return False
            
            return (mirror[0].progress() >= 1.0)
        
        return [(lambda: number_of_b(Temple)    >=   1, self.on_temple      ),
                (lambda: number_of_b(Temple)    >=   3, self.on_magic       ),
                (lambda: number_of_r(Sapphire)  >=   1, self.on_sapphire    ),
                (lambda: max_power()                  , self.on_done        )]
        
    def on_temple(self):
        self.message(['An awe inpiring structure! But will that make him content?'])
        self.flavor(['"AAAH, MY TEMPLE! NOW HIRE SOME PRIESTS TO WORSHIP ME! AND BUILD ME TWO MORE!'])
        self.flavor(['"DO THAT AND I SHALL SHOW YOU SOMETHING MAGICAL!"'])

        self.objective(['Build two more temples.'])
        
    def on_magic(self):
        self.flavor(['"We will see", you think. "Those sapphires will have great material value!"'])
        
        self.objective(['Mine a sapphire.'])
        
    def on_sapphire(self):
        self.allow(Mirror)
        
        self.message(['You can feel the gem\'s magic!'])
        self.flavor(['You hold the sapphire in your hand. "I cannot believe he showed you this." ',
                     '"Aqua?" You turn around. "He does not'])
        self.flavor(['know how to use their magic, but I do. They will provide me with ',
                     'enough power to strip him of his!" "What do I have'])
        self.flavor(['to do?" "Behind that waterfall is a cave with a pond. ',
                     'Put a sapphire in it and it will become a mirror of my power!"'])
        
        self.objective(['Build a mirror and max Aqua\'s power! Beware of retaliation strikes!'])
        
    def on_done(self):
        self.flavor(['The dark cloud descends on the mirror, trying to destroy it. ',
                     'Raw energy shoots out of it, hitting the cloud.'])
        self.flavor(['The cloud contracts into a dark crystal. Aqua appears. The crystal turns into ',
                     'liquid in her hand, then forms a human.'])
        self.flavor(['"What have you done! I am mortal again!" "You never were a god", '
                     'Aqua says. "Only a slave to the crystal\'s power."'])
        self.flavor(['"Now we will put you to justice for the crimes you did!", you say. ',
                     '"No. I will do that." She touches him. They vanish.'])
        
        self.objective(['And all live happily ever after! Thank you for playing!'])
        self.win()
    