'''The newbies' friend (I hope).'''

from river.scenario import *
from diwine import *


class Tutorial(Scenario):        
    @classmethod
    def name(cls):
        return 'Tutorial'

    @classmethod
    def next_scenario(self):
        return Diwine
    
    def city_name(self):
        return 'The Source Lake'
    
    def starting_tile(self):
        return self.map().tile(22, 38)
    
    def layout(self):
        return ['$$$$$$$$$$$$$$$$....~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '$$$$$$$$$$$$$.......~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '$$$$$.~~~~~........~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '$$$..~~~~~~~~~....~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '$$$~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '$$~~~~~...~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '$~~~~~.....~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '$~~~~........~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '~~~~..........~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~........~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '.~~~~~~.......~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '....~~~~~......~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '.....~~~~......~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '....~~~~~......~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~.......~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~.......~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~..........~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '..............~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '............~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '.......%%%..~~~~$..~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '.....%%%*%%~~~~.$=$.~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '...%%%%%%%~~~..$$$$..~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '..%%*%%.~~~~~~.$$$$$~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '...%%...~~~~~...=$.~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',
                '.......~~~~~~~~.~~~~~~...~~~~~~~~~~~~~~~~~~~~~~~~~',
                '......~~~~~~~~~~~~~~~.....~~~~~~~~~~~~~~~~~~~~~~~~',
                '...~~~~~~~~~~~....~~.......~~~~~~~~~~~~~~~~~~~~~~~',
                '..~~~~~~~~~~~........$$=$.~~~~~~~~~~~~~~~~~~~~~~~~',
                '.~~~~~~~~~~~..........$$$$=~~~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~.........$=$$$$~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~~~$=$......~$=$$=~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~~~~$$.......$$$$~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~~~~~~~.....=$$$~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~~~~~~~~~$....=$~~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~~~~~~~~.......r.~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~~~~~.$$.$.....r...~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~~~.$....$:rr.rr.~~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~~~....$.$.r:.rrr.$=~~~~~~~~~~~~~~~~~~~',
                '~~~~~~~~~~~~...$$.....:.....$$$$$~~~~~~~~~~~~~~~~~']

    def setup(self):
        self.allow(Farm)
        self.create_immigrants(4)
        
    def intro(self):
        self.flavor(['It has been two nights since he struck down this village. ',
                     'Chaos is still everywhere. You think of your pact with...'])
        self.flavor(['"You there", someone shouts at you. "Heading for the forest?" ',
                     'You recognize the face of mayor Tara and nod.'])
        self.flavor(['"Tell them to build a farm up there. Down at the fields ',
                     'everything is still in rubble and we\'ve lost all our wheat."'])

        self.objective(['Build a farm near the villagers at the bottom ',
                        'of the screen.'])

    def script(self):
        total_p     = lambda: len(self.city().persons())
        number_of_p = lambda x: len(self.city().persons_of_type(x))
        number_of_b = lambda x: len(self.city().buildings_of_type(x))
        
        return [(lambda: number_of_p('Builder')    ==  1, self.on_builder     ),
                (lambda: number_of_b(Farm)         ==  1, self.on_farm        ),
                (lambda: number_of_b(Sawmill)      ==  1, self.on_sawmill     ),
                (lambda: number_of_b(House)        ==  2, self.on_houses      ),
                (lambda: total_p()                 ==  6, self.on_immigrants  ),
                (lambda: number_of_b(Quarry)       ==  1, self.on_quarry      ),
                (lambda: number_of_b(Plaza)        ==  1, self.on_plaza       )]
        
    def on_builder(self):
        self.allow(Sawmill)
        
        name = self.city().persons_of_type('Builder')[0].name()
        
        self.message(['Well done! %s became a builder and is now gathering wood for the farm.' % name])
        self.flavor(['You approach %s with Tara\'s request. It takes some convincing. ' % name,
                     '"But we need to repair the sawmill!"'])
        self.flavor(['Eventually, you agree to see about the sawmill in exchange. You will ',
                     'need the wood it produces to get off this isle!'])
        
        self.objective(['Complete a sawmill in the forest close by.'])
        self.hint(['A person with a higher skill can do the job better. ',
                   'For builders, skill depends on "Body" and a bit on "Mind".'])
        
    def on_farm(self):
        self.message(['The farm is finally done! Every farm employs up to two persons. ',
                      'They will grow wheat to feed your people.'])
        
    def on_sawmill(self):
        name = random.choice(list(self.city().persons())).name()
        
        self.allow(House)
        self.message(['The sawmill is done! A lumberjack will fell trees and get you ',
                      'wood for more buildings.'])
        self.flavor(['After the sawmill is up, %s asks: "Now that we\'ve got ' % name,
                     'wood, can\'t we rebuild our homes?"'])
        self.flavor(['The others agree. "Better here than amidst that forsaken ruins!"'])
        
        self.objective(['Build two houses.'])
    
    def on_houses(self):
        name = random.choice(list(self.city().persons())).name()
        
        self.message(['The houses are done! Your people have now a safe place to sleep.'])
        self.flavor(['Everyone cheers as the second house becomes completed. ',
                     '%s says: "Maybe we should build another one?' % name])
        self.flavor(['Our friends still in that ruin would surely appreciate it!"'])
        
        self.objective(['Build at least another house and wait for immigrants.'])
        self.hint(['Every house provides space for two people. Empty space ',
                   'will be filled through immigration.'])

    def on_immigrants(self):
        self.allow(Bridge)
        self.allow(Quarry)
        
        self.message(['Two old friends join you! Well done!'])
        self.flavor(['Your small village welcomes the new arrivals. It seems ',
                     'that things are only slowly improving down there.'])
        self.flavor(['You will need more durable buildings. A quarry ',
                     'on the rock in the north-west would allow them.'])
        
        self.objective(['Build at least one quarry on the rock in the north-west. Continue ',
                        'improving your village core.'])
        self.hint(['Use bridges to reach the mainland. ',
                   'Maybe a sawmill on the small island would be handy?'])

    def on_quarry(self):
        self.allow(Road)
        self.allow(Plaza)
        
        name = random.choice(list(self.city().persons())).name()
        
        self.message(['The new quarry secures your stone supply!'])
        self.flavor(['Soon after the quarry is finished, the first ',
                     'complaints about the lack of social'])
        self.flavor(['life arrive. "We need a plaza to hang out on!", ',
                     '%s proclaims. Many concur.' % name])
        
        self.objective(['Build a plaza near your village core. Continue building ',
                        'your housing and resource buildings.'])
        self.hint(['Building roads makes your people go faster, ',
                   'especially in difficult terrain.'])

    def on_plaza(self):
        self.message(['Your people greatly enjoy conversations on the new plaza!'])
        self.flavor(['It is time to say goodbye. You step off the island and ',
                     'are prepared to follow where ever the god will lead you'])
        self.flavor(['in exchange for him sparing this island. This is the deal ',
                     'that you struck and this you need to follow through with.'])
        
        self.objective(['You win!'])
        self.hint(['Click "Continue" on the top right to start the first ',
                   'scenario of the campaign.'])
        
        self.win()
    