'''All terrain in River.'''

from concepts.color import *

from goods import *


class TerrainType(object):
    def color(self):
        return Color(255, 0, 0)

    def difficulty(self):
        return None
    
    def is_passable(self):
        return (self.difficulty() is not None)
    
    def __str__(self):
        return self.__class__.__name__

    
class Grass(TerrainType):
    def color(self):
        return Color(0, 156, 64)

    def difficulty(self):
        return 1
    

class Forest(TerrainType):
    def color(self):
        return Color(0, 84, 32)

    def difficulty(self):
        return 2
    

class Rock(TerrainType):
    def color(self):
        return Color(96, 96, 96)

    def difficulty(self):
        return 2
    

class MarbleRock(TerrainType):
    def color(self):
        return Color(222, 222, 222)

    def difficulty(self):
        return 2
    
    def __str__(self):
        return 'Marble'
    

class Mountain(TerrainType):
    def __init__(self):
        self._resource = Ore
        
    def resource(self):
        return self._resource
    
    def deposit(self, resource):
        self._resource = resource
        
    def color(self):
        return Color(48, 48, 48)

    def difficulty(self):
        return 5
    

class MagicalMountain(Mountain):
    def __init__(self):
        self._resource = Sapphire
        
    def color(self):
        return Color(48, 48, 128)

    def difficulty(self):
        return 5
    
    def __str__(self):
        return 'Mountain'
    
    
class River(TerrainType):
    def color(self):
        return Color(0, 32, 164)

    def difficulty(self):
        return 5
    
    
class Waterfall(TerrainType):
    def color(self):
        return Color(192, 220, 255)

    def difficulty(self):
        return 20


class Outside(TerrainType):
    def color(self):
        return Color(0, 0, 0)

    def is_land(self):
        return False

    def difficulty(self):
        return None
