'''Time component that makes events happen.'''

from threading import Timer


class Time(object):
    def __init__(self, speed):
        self._speed      = speed
        self._timer      = None
        self._running    = False
        self._events     = {}
        self._one_time   = {}
        self._conditions = []
        self._ui_events  = []
        self._tick       = 0
        
    def speed(self):
        return self._speed
    
    def set_speed(self, speed):
        self._speed = speed
    
    def create_timer(self):
        self._timer = Timer(self.speed(), self.do_tick)
        
    def timer(self):
        return self._timer
    
    def running(self):
        return self._running
    
    def events(self, ticks):
        return list(self._events.get(ticks, []))
    
    def one_time_events(self, tick):
        return list(self._one_time.get(tick, []))

    def check_conditions(self):
        todo = []
        keep = []
        
        for condition, event, one_time in self._conditions:
            if condition():
                todo.append(event)
                
                if not one_time:
                    keep.append((condition, event, one_time))
            else:
                keep.append((condition, event, one_time))
                
        self._conditions = keep
        
        return todo
    
    def next_ui_event(self):
        try:
            return self._ui_events.pop(0)
        except IndexError:
            return None
    
    def add_ui_event(self, event):
        self._ui_events.append(event)
        
    def tick(self):
        return self._tick
    
    def do_every(self, ticks, event):
        if not self._events.has_key(ticks):
            self._events[ticks] = []
            
        self._events[ticks].append(event)
    
    def do_at(self, tick, event):
        if not self._one_time.has_key(tick):
            self._one_time[tick] = []

        self._one_time[tick].append(event)
    
    def do_after(self, tick, event):
        self.do_at(tick + self.tick(), event)
    
    def do_when(self, condition, event, one_time = True):
        self._conditions.append((condition, event, one_time))
        
    def start(self):
        if self.running():
            return

        self._running = True
        
        self.create_timer()
        self.timer().start()
    
    def stop(self):
        if not self.running():
            return
        
        self.timer().cancel()
        
        self._running = False
        self._timer = None
    
    def do_tick(self):
        self._tick += 1
        
        while True:
            event = self.next_ui_event()
            
            if event is None:
                break
            
            event()
    
        for key in self._events:
            if self.tick() % key != 0:
                continue
            
            for event in self.events(key):
                event()
        
        for event in self.one_time_events(self.tick()):
            event()
        
        for event in self.check_conditions():
            event()
        
        if self.running():
            self.create_timer()
            self.timer().start()
        